@@ -14,9 +14,14 @@ use YATT::Lite::MFields qw/cf_namespace cf_debug_cgen cf_no_lineinfo cf_check_li
 	      cf_die_in_error cf_error_handler
 	      cf_special_entities
 	      cf_lcmsg_sink
+              cf_match_argsroute_first
+              cf_body_argument
+              cf_body_argument_type
+
+              cf_stash_unknown_params_to
+	      cf_prefer_call_for_entity
 
 	      n_compiles
-	      cgen_class
 	    /;
 use YATT::Lite::Util;
 use YATT::Lite::Constants;
@@ -33,7 +38,9 @@ use YATT::Lite::Breakpoint ();
   use YATT::Lite::Types
     ([Part => -base => MY->Item
       , -fields => [qw(toks arg_dict arg_order
+                       decllist
 		       cf_namespace cf_kind cf_folder cf_data
+                       cf_decl
 		       cf_implicit cf_suppressed
 		       cf_startln cf_bodyln cf_endln
 		       cf_startpos cf_bodypos cf_bodylen
@@ -42,11 +49,18 @@ use YATT::Lite::Breakpoint ();
       , -constants => [[public => 0]]
       , [Widget => -fields => [qw(tree var_dict has_required_arg)]
 	 , [Page => (), -constants => [[public => 1]]]]
-      , [Action => (), -constants => [[public => 1]]]
-      , [Data => ()]]
+      , [Action => (), -constants => [[public => 1],
+                                      [item_category => 'do'],
+                                    ]]
+      , [Data => ()]
+      , [Entity => ()
+         , -constants => [[item_category => 'entity']]
+       ]
+    ]
 
      , [Template => -base => MY->File
 	, -alias => 'vfs_file'
+	, -constants => [[can_generate_code => 1]]
 	, -fields => [qw(product parse_ok cf_mtime cf_utf8 cf_age
 			 cf_usage cf_constants
 			 cf_ignore_trailing_newlines
@@ -56,10 +70,65 @@ use YATT::Lite::Breakpoint ();
      , [ParsingState => -fields => [qw(startln endln
 				       startpos curpos
 				       cf_path
-				       )]]
+                                    )]]
+
+     , [AbstParser => -fields => [qw(cf_body_argument
+                                     cf_body_argument_type
+                                  )]]
     );
 
-  # folder の weaken は parser がしてる。
+  sub YATT::Lite::Core::Part::public_name {
+    (my Part $part) = @_;
+    $part->{cf_name};
+  }
+  sub YATT::Lite::Core::Part::decl_kind {
+    (my Part $part) = @_;
+    join(":", $part->{cf_namespace}, $part->{cf_decl});
+  }
+  sub YATT::Lite::Core::Part::syntax_keyword {
+    (my Part $part) = @_;
+    join(" ", $part->decl_kind, $part->syntax_name);
+  }
+  *YATT::Lite::Core::Part::syntax_name = *YATT::Lite::Core::Part::public_name;
+  *YATT::Lite::Core::Part::syntax_name = *YATT::Lite::Core::Part::public_name;
+  sub YATT::Lite::Core::Widget::syntax_name {
+    (my Widget $widget) = @_;
+    $widget->{cf_decl} eq 'args' ? () : $widget->{cf_name};
+  }
+  sub YATT::Lite::Core::Action::syntax_name {
+    (my Action $action) = @_;
+    $action->{cf_name} eq '' ? q{''} : $action->{cf_name};
+  }
+
+  sub YATT::Lite::Core::Part::method_name {...}
+  sub YATT::Lite::Core::Widget::method_name {
+    (my Widget $widget) = @_;
+    "render_$widget->{cf_name}";
+  }
+  sub YATT::Lite::Core::Action::method_name {
+    (my Action $action) = @_;
+    "do_$action->{cf_name}";
+  }
+  sub YATT::Lite::Core::Action::item_key {
+    (my Action $action) = @_;
+    "do\0$action->{cf_name}";
+  }
+
+  sub YATT::Lite::Core::Entity::method_name {
+    (my Entity $entity) = @_;
+    "entity_$entity->{cf_name}";
+  }
+  sub YATT::Lite::Core::Entity::item_key {
+    (my Entity $entity) = @_;
+    "entity\0$entity->{cf_name}";
+  }
+
+  sub YATT::Lite::Core::Part::configure_folder {
+    (my Part $part, my Folder $folder) = @_;
+    Scalar::Util::weaken($part->{cf_folder} = $folder);
+    # die "Can't weaken!" unless Scalar::Util::isweak($part->{cf_folder});
+  }
+
 #  sub YATT::Lite::Core::Part::source {
 #    (my Part $part) = @_;
 #    join "", map {ref $_ ? "\n" x $$_[0] : $_} @{$part->{source}};
@@ -70,6 +139,7 @@ use YATT::Lite::Breakpoint ();
   }
   sub YATT::Lite::Core::Template::list_parts {
     (my Template $self, my $type) = @_;
+    return unless $self->{partlist};
     return @{$self->{partlist}} unless defined $type;
     grep { UNIVERSAL::isa($_, $type) } @{$self->{partlist}}
   }
@@ -87,6 +157,13 @@ use YATT::Lite::Breakpoint ();
     }
     $tmpl->source_region($node->[NODE_BODY_BEGIN], $node->[NODE_BODY_END]);
   }
+  sub YATT::Lite::Core::Template::node_outer_source {
+    (my Template $tmpl, my $node) = @_;
+    unless (ref $node eq 'ARRAY') {
+      confess "Node is not an ARRAY";
+    }
+    $tmpl->source_region($node->[NODE_BEGIN], $node->[NODE_BODY_END]);
+  }
   sub YATT::Lite::Core::Template::source_region {
     (my Template $tmpl, my ($begin, $end)) = @_;
     $tmpl->source_substr($begin, $end - $begin);
@@ -102,24 +179,47 @@ use YATT::Lite::Breakpoint ();
   }
 
   sub YATT::Lite::Core::Part::reorder_hash_params {
-    (my Widget $widget, my ($params)) = @_;
+    (my Widget $widget, my ($orig_params)) = @_;
+    return unless $orig_params;
+    return @$orig_params if ref $orig_params eq 'ARRAY';
+    my $params = +{%$orig_params};
     my @params;
     foreach my $name (map($_ ? @$_ : (), $widget->{arg_order})) {
       push @params, delete $params->{$name};
     }
-    if (keys %$params) {
-      die "Unknown args for $widget->{cf_name}: " . join(", ", keys %$params);
+    if (my @unknown = grep {/^[a-z]\w*$/i} keys %$params) {
+      die "Unknown args for $widget->{cf_name}: " . join(", ", @unknown)
+	. "\n";
     }
     wantarray ? @params : \@params;
   }
-
-  sub YATT::Lite::Core::Part::reorder_cgi_params {
-    (my Widget $widget, my ($cgi, $list)) = @_;
+}
+{
+  sub reorder_cgi_params {
+    (my MY $self, my Widget $widget, my ($cgi, $list)) = @_;
     $list ||= [];
+    my $stash;
+    if ($self->{cf_stash_unknown_params_to}) {
+      $stash = $cgi->stash->{$self->{cf_stash_unknown_params_to}} //= +{};
+    }
     foreach my $name ($cgi->param) {
       next unless $name =~ /^[a-z]\w*$/i;
-      my $argdecl = $widget->{arg_dict}{$name}
-	or die "Unknown args for widget '$widget->{cf_name}': $name";
+      my $argdecl = $widget->{arg_dict}{$name} or do {
+        if ($stash) {
+          push @{$stash->{$name}}, $cgi->multi_param($name);
+          next;
+        } else {
+          my $wname = $widget->{cf_name}
+            ? " for widget '$widget->{cf_name}'" : "";
+          die "Unknown args$wname: $name\n";
+        }
+      };
+      if ($argdecl->is_unsafe_param) {
+        if ($stash) {
+          push @{$stash->{$name}}, $cgi->multi_param($name);
+        }
+        next;
+      }
       my @value = $cgi->multi_param($name);
       $list->[$argdecl->argno] = $argdecl->type->[0] eq 'list'
 	? \@value : $value[0];
@@ -171,20 +271,18 @@ sub declare_base {
     nonempty(my $fn = $vfs->node_value($att))
       or $vfs->synerror($state, q{base spec is empty!});
 
-    my Folder $dirobj = $tmpl->dirobj;
-
     if ($vfs->{on_memory}) {
       my $o = $vfs->find_file($fn)
 	or $vfs->synerror($state, q{No such base path: %s}, $fn);
       push @$base, $o;
     } else {
-      defined(my $realfn = $vfs->resolve_path_from($dirobj, $fn))
+      defined(my $realfn = $vfs->resolve_path_from($tmpl, $fn))
 	or $vfs->synerror($state, q{Can't find object path for: %s}, $fn);
 
       -e $realfn
 	or $vfs->synerror($state, q{No such base path: %s}, $realfn);
-      my $kind = -d $realfn ? 'dir' : 'file';
-      push @$base, $vfs->create($kind => $realfn, parent => $dirobj);
+
+      push @$base, $vfs->find_neighbor_type(undef, $realfn);
     }
   }
 }
@@ -220,7 +318,11 @@ sub synerror {
     # $self->{parser} ||=
       $self->Parser->new
 	(vfs => $self, $self->cf_delegate
-	 (qw(namespace special_entities)
+	 (qw(namespace special_entities
+             match_argsroute_first
+             body_argument
+             body_argument_type
+          )
 	  , [debug_parser => 'debug']
 	  , [tmpl_encoding => 'encoding']
 	 )
@@ -228,10 +330,12 @@ sub synerror {
 	 , @_);
   }
   sub ensure_parsed {
-    (my MY $self, my Widget $widget) = @_;
-    $self->get_parser->parse_body($widget->{cf_folder});
-    # $self->get_parser->parse_widget($widget)
-    @{$widget->{tree}};
+    (my MY $self, my Part $part) = @_;
+    my $parser = $self->get_parser;
+    my Template $tmpl = $part->{cf_folder};
+    return if $tmpl->{parse_ok};
+    $parser->parse_decllist_entities($tmpl);
+    $parser->parse_body($tmpl);
   }
   sub render {
     my MY $self = shift;
@@ -249,7 +353,7 @@ sub synerror {
     }
 
     my @args = do {
-      unless (defined $args and $part->isa(MY->Widget)) {
+      if (not defined $args) {
 	();
       } elsif (ref $args eq 'ARRAY') {
 	@$args
@@ -285,6 +389,33 @@ sub synerror {
     wantarray ? ($part, $tmpl) : $part;
   }
 
+  sub find_part_renderer {
+    (my MY $self, my ($widgetPath, %opts)) = @_;
+    my $ignore_error = delete $opts{ignore_error};
+
+    my @wpath = ref $widgetPath ? @$widgetPath : split ":", $widgetPath;
+
+    my $part = $self->find_part_from($self->{root}, @wpath ? @wpath : '')
+      or ($ignore_error and return)
+      or croak "No such widget: ".join(":", @wpath);
+
+    my $tmpl = $part->cget('folder');
+
+    my $path = $tmpl->cget('path');
+
+    my $method = "render_".$part->cget('name');
+
+    my $pkg = $self->find_product(perl => $tmpl)
+      or ($ignore_error and return)
+	or croak "Can't compile template file: $path";
+
+    my $sub = $pkg->can($method)
+      or ($ignore_error and return)
+	or croak "Can't extract $method from file: $path";
+
+    ($part, $sub, $pkg);
+  }
+
   sub find_part_handler {
     (my MY $self, my $nameSpec, my %opts) = @_;
     my $ignore_error = delete $opts{ignore_error};
@@ -292,33 +423,47 @@ sub synerror {
       = ref $nameSpec ? @$nameSpec : $nameSpec;
 
     $partName ||= $self->{cf_index_name};
-    $kind //= 'page';
-    $pureName //= '';
 
-    my ($itemKey, $method) = $self->can("_itemKey_$kind")->($self, $pureName);
+    my Template $tmpl = do {
+      if (UNIVERSAL::isa($self->{root}, Template)) {
+        # Special case.
+        # XXX: Should add action tests for this case.
+        $self->{root};
 
-    (my Template $tmpl, my Part $part);
-
-    if (UNIVERSAL::isa($self->{root}, Template)) {
-      # Special case.
-      $tmpl = $self->{root};
-
-      $part = $tmpl->{Item}{$partName}
-	or ($ignore_error and return)
-	  or croak "No such item in template: $partName";
-
-      $method = "render_$partName";
-
-    } else {
-      # General container case.
-      $tmpl = $self->find_file($partName)
-	or ($ignore_error and return)
+      } else {
+        # General container case.
+        $self->find_file($partName)
+          or ($ignore_error and return)
 	  or croak "No such template file: $partName";
-      $part = $tmpl->{Item}{$itemKey}
-	or ($ignore_error and return)
-	  or croak "No such $kind in file $partName: $pureName";
-    }
+      }
+    };
 
+    (my Part $part, my $method) = do {
+      (my Part $p, my $meth);
+      if (not defined $kind and not defined $pureName) {
+        foreach my $k (qw(page action)) {
+          (my $itemKey, $meth) = $self->can("_itemKey_$k")->($self, '');
+          $p = $tmpl->{Item}{$itemKey}
+            and last;
+        }
+      }
+
+      if ($p) {
+        ($p, $meth);
+      } else {
+
+        $kind //= 'page';
+        $pureName //= '';
+
+        my ($itemKey, $meth) = $self->can("_itemKey_$kind")->($self, $pureName);
+
+        $p = $tmpl->{Item}{$itemKey} || $self->find_part_from($tmpl, $itemKey)
+          or ($ignore_error and return)
+          or croak "No such $kind in file $partName: $pureName";
+
+        ($p, $meth);
+      };
+    };
 
     my $pkg = $self->find_product(perl => $tmpl)
       or ($ignore_error and return)
@@ -332,7 +477,29 @@ sub synerror {
   }
 
   sub _itemKey_page { shift; ($_[0], "render_$_[0]") }
-  sub _itemKey_action { shift; ("do_$_[0]") x 2; }
+  sub _itemKey_action { shift; ("do\0$_[0]", "do_$_[0]"); }
+
+  #
+  # Action name => sub {}
+  #
+  sub add_root_action_handler {
+    (my MY $self, my ($name, $sub, $callinfo)) = @_;
+    my Folder $root = $self->{root};
+
+    my ($callpack, $filename, $lineno) = @$callinfo;
+
+    # XXX: This means do_$A.yatt will conflict with "Action $A" in .htyattrc.pl
+    my $action_name = "do_$name";
+
+    *{globref($root->{cf_entns}, $action_name)} = $sub;
+
+    $root->{Item}{"do\0$name"}
+      = $self->Action->new(name => $action_name, kind => 'action'
+			   , folder => $root
+			   , startln => $lineno
+			 );
+
+  }
 
   sub find_renderer {
     my MY $self = shift;
@@ -344,15 +511,7 @@ sub synerror {
   # DirHandler INST 固有 CGEN_perl の生成
   sub get_cgen_class {
     (my MY $self, my $type) = @_;
-    my $sub = $self->can("cgen_$type")
-      || carp "Unknown product type: $type";
-    $self->{cgen_class}{$type} ||= do {
-      my $cg_base = $sub->();
-      # XXX: ref($facade) が INST 固有に成ってなかったら？
-      my $instpkg = ref($self->{cf_facade})."::CGEN_$type";
-      ckeval(qq|package $instpkg; use base qw($cg_base)|);
-      $instpkg;
-    };
+    $self->{cf_facade}->get_cgen_class($type);
   }
 
   # XXX: Action only コンパイルは？
@@ -361,35 +520,41 @@ sub synerror {
     my ($type, $kind) = ref $spec ? @$spec : $spec;
     # local $YATT = $self;
     unless ($tmpl->{product}{$type}) {
-      my $cg_class = $self->get_cgen_class($type);
-      my $cgen = $cg_class->new
-	(vfs => $self
-	 , $self->cf_delegate(qw(no_lineinfo check_lineno only_parse
-				 lcmsg_sink))
-	 , parser => $self->get_parser
-	 , sink => $opts{sink} || sub {
-	   my ($info, @script) = @_;
-	   if (not $self->{cf_debug_cgen}) {
-	   } else {
-	     my Template $real = $info->{folder};
-	     print STDERR "# compiling $type code of $real->{cf_path}\n";
-	     if ($self->{cf_debug_cgen} >= 2) {
-	       print STDERR "#--BEGIN--\n";
-	       print STDERR @script, "\n";
-	       print STDERR "#--END--\n\n"
-	     }
-	   }
-	   #
-	   $self->{n_compiles}++;
-
-	   ckeval(@script);
-	 });
+      my $cgen = $self->build_cgen_of($type, \%opts);
       # 二重生成防止のため、代入自体は ensure_generated の中で行う。
       $cgen->ensure_generated($spec => $tmpl);
     };
     $tmpl->{product}{$type};
   }
 
+  sub build_cgen_of {
+    (my MY $self, my $cgenSpec, my $opts) = @_;
+    my ($type, $cg_class) = lexpand($cgenSpec);
+    $cg_class //= $self->get_cgen_class($type);
+    $cg_class->new
+      (vfs => $self
+       , $self->cf_delegate(qw(no_lineinfo check_lineno only_parse
+                               prefer_call_for_entity
+                               lcmsg_sink))
+       , parser => $self->get_parser
+       , sink => $opts->{sink} || sub {
+         my ($info, @script) = @_;
+         if ($self->{cf_debug_cgen}) {
+           my Template $real = $info->{folder};
+           print STDERR "# compiling @{[$type//'undef']} code of @{[$real->{cf_path}//'undef']}\n";
+           if ($self->{cf_debug_cgen} >= 2) {
+             print STDERR "#--BEGIN--\n";
+             print STDERR @script, "\n";
+             print STDERR "#--END--\n\n"
+           }
+         }
+         #
+         $self->{n_compiles}++;
+
+         ckeval(@script);
+       })
+  }
+
   #
   # extract_lcmsg
   #  - filelist is a list(or scalar) of filename or item name(no ext).
@@ -458,15 +623,25 @@ sub synerror {
     if ($tmpl->{cf_path}) {
       printf STDERR "template_refresh(%s)\n", $tmpl->{cf_path} if DEBUG_REBUILD;
       my $mtime = stat_mtime($tmpl->{cf_path});
-      unless (defined $mtime) {
+      if (not defined $mtime) {
 	printf STDERR " => deleted\n" if DEBUG_REBUILD;
 	return; # XXX: ファイルが消された
-      } elsif (defined $tmpl->{cf_mtime} and $tmpl->{cf_mtime} >= $mtime) {
-	printf STDERR " => not updated.\n" if DEBUG_REBUILD;
+      } elsif (not defined $tmpl->{cf_mtime}) {
+        if (DEBUG_REBUILD) {
+          printf STDERR " => found new. mtime($mtime) for tmpl=$tmpl\n";
+        }
+      } elsif ($tmpl->{cf_mtime} >= $mtime) {
+	if (DEBUG_REBUILD) {
+	  printf STDERR " => use cached. mtime(was=$tmpl->{cf_mtime}"
+	    .", now=$mtime) for tmpl=$tmpl\n";
+	}
 	$self->refresh_deps_for($tmpl) if $self->{cf_always_refresh_deps};
 	return; # timestamp は、キャッシュと同じかむしろ古い
+      } else {
+        if (DEBUG_REBUILD) {
+          printf STDERR " => found update. mtime($mtime) for tmpl=$tmpl\n";
+        }
       }
-      printf STDERR " => found update\n" if DEBUG_REBUILD;
       $tmpl->{cf_mtime} = $mtime;
       my $parser = $self->get_parser;
       # decl のみ parse.
@@ -495,16 +670,26 @@ sub synerror {
     $tmpl;
   }
   sub YATT::Lite::Core::Widget::fixup {
-    (my Widget $widget, my Template $tmpl, my $parser) = @_;
+    (my Widget $widget, my Template $tmpl, my AbstParser $parser) = @_;
     foreach my $argName (@{$widget->{arg_order}}) {
       $widget->{has_required_arg} = 1
 	if $widget->{arg_dict}{$argName}->is_required;
     }
-    $widget->{arg_dict}{body} ||= do {
+    $widget->{arg_dict}{$parser->{cf_body_argument}} ||= do {
+      my ($type, @dflag_default) = $parser->parse_type_dflag_default(
+        $parser->{cf_body_argument_type}
+      );
+
       # lineno も入れるべきかも。 $widget->{cf_bodyln} あたり.
-      my $var = $parser->mkvar_at(undef, code => 'body'
+      my $var = $parser->mkvar_at(undef
+                                  , $type
+                                  , $parser->{cf_body_argument}
 				  , scalar @{$widget->{arg_order} ||= []});
-      push @{$widget->{arg_order}}, 'body';
+      # body_argument の印を付ける。public からは受理しないように.
+      $var->mark_body_argument;
+      $parser->set_dflag_default_to($var, @dflag_default);
+
+      push @{$widget->{arg_order}}, $parser->{cf_body_argument};
       $var;
     };
   }
@@ -516,11 +701,6 @@ sub synerror {
   }
 }
 
-sub find_template_from_package {
-  (my MY $self, my $pkg) = @_;
-  $self->{pkg2folder}{$pkg};
-}
-
 use YATT::Lite::Breakpoint ();
 YATT::Lite::Breakpoint::break_load_core();
 
